@@ -82,6 +82,8 @@ gboolean actions_register(const gchar *name,
 
 gboolean actions_set_shutdown(const gchar *name,
                               ObActionsShutdownFunc shutdown);
+gboolean actions_set_modifies_focused_window(const gchar *name,
+                                             gboolean modifies);
 
 ObActionsAct* actions_parse(xmlNodePtr node);
 ObActionsAct* actions_parse_string(const gchar *name);
