@@ -6,8 +6,9 @@ static gboolean run_func_ur(ObActionsData *data, gpointer options);
 
 void action_shadelowerraise_startup()
 {
-    actions_register("ShadeLower", NULL, NULL, run_func_sl, NULL, NULL);
-    actions_register("UnshadeRaise", NULL, NULL, run_func_ur, NULL, NULL);
+    /* 3.4-compatibility */
+    actions_register("ShadeLower", NULL, NULL, run_func_sl);
+    actions_register("UnshadeRaise", NULL, NULL, run_func_ur);
 }
 
 /* Always return FALSE because its not interactive */
