@@ -26,10 +26,10 @@
 #include "group.h"
 #include "focus_cycle.h"
 #include "screen.h"
-#include "prop.h"
 #include "keyboard.h"
 #include "focus.h"
 #include "stacking.h"
+#include "obt/prop.h"
 
 #include <X11/Xlib.h>
 #include <glib.h>
@@ -52,7 +52,7 @@ void focus_shutdown(gboolean reconfig)
     if (reconfig) return;
 
     /* reset focus to root */
-    XSetInputFocus(ob_display, PointerRoot, RevertToNone, CurrentTime);
+    XSetInputFocus(obt_display, PointerRoot, RevertToNone, CurrentTime);
 }
 
 static void push_to_top(ObClient *client)
@@ -74,7 +74,7 @@ void focus_set_client(ObClient *client)
     Window active;
 
     ob_debug_type(OB_DEBUG_FOCUS,
-                  "focus_set_client 0x%lx\n", client ? client->window : 0);
+                  "focus_set_client 0x%lx", client ? client->window : 0);
 
     if (focus_client == client)
         return;
@@ -98,9 +98,12 @@ void focus_set_client(ObClient *client)
     /* set the NET_ACTIVE_WINDOW hint, but preserve it on shutdown */
     if (ob_state() != OB_STATE_EXITING) {
         active = client ? client->window : None;
-        PROP_SET32(RootWindow(ob_display, ob_screen),
-                   net_active_window, window, active);
+        OBT_PROP_SET32(obt_root(ob_screen), NET_ACTIVE_WINDOW, WINDOW, active);
     }
+
+    /* when focus is moved to a new window, the last_user_time timestamp would
+       no longer be valid, as it applies for the focused window */
+    event_reset_user_time();
 }
 
 static ObClient* focus_fallback_target(gboolean allow_refocus,
@@ -111,18 +114,18 @@ static ObClient* focus_fallback_target(gboolean allow_refocus,
     GList *it;
     ObClient *c;
 
-    ob_debug_type(OB_DEBUG_FOCUS, "trying pointer stuff\n");
+    ob_debug_type(OB_DEBUG_FOCUS, "trying pointer stuff");
     if (allow_pointer && config_focus_follow)
         if ((c = client_under_pointer()) &&
             (allow_refocus || client_focus_target(c) != old) &&
             (client_normal(c) &&
              client_focus(c)))
         {
-            ob_debug_type(OB_DEBUG_FOCUS, "found in pointer stuff\n");
+            ob_debug_type(OB_DEBUG_FOCUS, "found in pointer stuff");
             return c;
         }
 
-    ob_debug_type(OB_DEBUG_FOCUS, "trying the focus order\n");
+    ob_debug_type(OB_DEBUG_FOCUS, "trying the focus order");
     for (it = focus_order; it; it = g_list_next(it)) {
         c = it->data;
         /* fallback focus to a window if:
@@ -133,17 +136,19 @@ static ObClient* focus_fallback_target(gboolean allow_refocus,
            3. it is not shaded
         */
         if ((allow_omnipresent || c->desktop == screen_desktop) &&
-            focus_valid_target(c, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE) &&
+            focus_valid_target(c, screen_desktop,
+                               TRUE, FALSE, FALSE, TRUE, FALSE, FALSE,
+                               FALSE) &&
             !c->shaded &&
             (allow_refocus || client_focus_target(c) != old) &&
             client_focus(c))
         {
-            ob_debug_type(OB_DEBUG_FOCUS, "found in focus order\n");
+            ob_debug_type(OB_DEBUG_FOCUS, "found in focus order");
             return c;
         }
     }
 
-    ob_debug_type(OB_DEBUG_FOCUS, "trying a desktop window\n");
+    ob_debug_type(OB_DEBUG_FOCUS, "trying a desktop window");
     for (it = focus_order; it; it = g_list_next(it)) {
         c = it->data;
         /* fallback focus to a window if:
@@ -153,11 +158,12 @@ static ObClient* focus_fallback_target(gboolean allow_refocus,
            a splashscreen or a desktop window (save the desktop as a
            backup fallback though)
         */
-        if (focus_valid_target(c, TRUE, FALSE, FALSE, FALSE, TRUE, FALSE) &&
+        if (focus_valid_target(c, screen_desktop,
+                               TRUE, FALSE, FALSE, TRUE, FALSE, TRUE, FALSE) &&
             (allow_refocus || client_focus_target(c) != old) &&
             client_focus(c))
         {
-            ob_debug_type(OB_DEBUG_FOCUS, "found a desktop window\n");
+            ob_debug_type(OB_DEBUG_FOCUS, "found a desktop window");
             return c;
         }
     }
@@ -191,8 +197,8 @@ void focus_nothing(void)
     focus_set_client(NULL);
 
     /* when nothing will be focused, send focus to the backup target */
-    XSetInputFocus(ob_display, screen_support_win, RevertToPointerRoot,
-                   event_curtime);
+    XSetInputFocus(obt_display, screen_support_win, RevertToPointerRoot,
+                   event_time());
 }
 
 void focus_order_add_new(ObClient *c)
@@ -285,8 +291,9 @@ static gboolean focus_target_has_siblings(ObClient *ft,
         ObClient *c = it->data;
         /* check that it's not a helper window to avoid infinite recursion */
         if (c != ft && c->type == OB_CLIENT_TYPE_NORMAL &&
-            focus_valid_target(c, TRUE, iconic_windows, all_desktops,
-                               FALSE, FALSE, FALSE))
+            focus_valid_target(c, screen_desktop,
+                               TRUE, iconic_windows, all_desktops,
+                               TRUE, FALSE, FALSE, FALSE))
         {
             return TRUE;
         }
@@ -295,9 +302,11 @@ static gboolean focus_target_has_siblings(ObClient *ft,
 }
 
 gboolean focus_valid_target(ObClient *ft,
+                            guint    desktop,
                             gboolean helper_windows,
                             gboolean iconic_windows,
                             gboolean all_desktops,
+                            gboolean nonhilite_windows,
                             gboolean dock_windows,
                             gboolean desktop_windows,
                             gboolean user_request)
@@ -315,9 +324,12 @@ gboolean focus_valid_target(ObClient *ft,
 
        do this check first because it will usually filter out the most
        windows */
-    ok = (all_desktops || ft->desktop == screen_desktop ||
+    ok = (all_desktops || ft->desktop == desktop ||
           ft->desktop == DESKTOP_ALL);
 
+    /* if we only include hilited windows, check if the window is */
+    ok = ok && (nonhilite_windows || ft->demands_attention);
+
     /* the window can receive focus somehow */
     ok = ok && (ft->can_focus || ft->focus_notify);
 
@@ -363,9 +375,11 @@ gboolean focus_valid_target(ObClient *ft,
     {
         ObClient *cft = client_focus_target(ft);
         ok = ok && (ft == cft || !focus_valid_target(cft,
+                                                     screen_desktop,
                                                      TRUE,
                                                      iconic_windows,
                                                      all_desktops,
+                                                     nonhilite_windows,
                                                      dock_windows,
                                                      desktop_windows,
                                                      FALSE));
