@@ -90,7 +90,16 @@ void menu_startup(gboolean reconfig)
             loaded = TRUE;
             obt_xml_tree_from_root(menu_parse_inst);
             obt_xml_close(menu_parse_inst);
-        } else
+        }
+        else if (obt_xml_load_file(menu_parse_inst,
+                                   it->data,
+                                   "openbox_menu"))
+        {
+            loaded = TRUE;
+            obt_xml_tree_from_root(menu_parse_inst);
+            obt_xml_close(menu_parse_inst);
+        }
+        else
             g_message(_("Unable to find a valid menu file \"%s\""),
                       (const gchar*)it->data);
     }
@@ -115,10 +124,11 @@ void menu_shutdown(gboolean reconfig)
     obt_xml_instance_unref(menu_parse_inst);
     menu_parse_inst = NULL;
 
-    client_list_menu_shutdown(reconfig);
-    client_list_combined_menu_shutdown(reconfig);
-
     menu_frame_hide_all();
+
+    client_list_combined_menu_shutdown(reconfig);
+    client_list_menu_shutdown(reconfig);
+
     g_hash_table_destroy(menu_hash);
     menu_hash = NULL;
 }
@@ -278,7 +288,7 @@ static void parse_menu_item(xmlNodePtr node,  gpointer data)
         /* Don't try to extract "icon" attribute if icons in user-defined
            menus are not enabled. */
 
-        if (obt_xml_attr_string(node, "label", &label)) {
+        if (obt_xml_attr_string_unstripped(node, "label", &label)) {
             xmlNodePtr c;
             GSList *acts = NULL;
 
@@ -287,7 +297,7 @@ static void parse_menu_item(xmlNodePtr node,  gpointer data)
                 ObActionsAct *action = actions_parse(c);
                 if (action)
                     acts = g_slist_append(acts, action);
-                c = obt_xml_find_node(node->next, "action");
+                c = obt_xml_find_node(c->next, "action");
             }
             e = menu_add_normal(state->parent, -1, label, acts, TRUE);
             
@@ -313,7 +323,7 @@ static void parse_menu_separator(xmlNodePtr node, gpointer data)
     if (state->parent) {
         gchar *label;
 
-        if (!obt_xml_attr_string(node, "label", &label))
+        if (!obt_xml_attr_string_unstripped(node, "label", &label))
             label = NULL;
 
         menu_add_separator(state->parent, -1, label);
@@ -333,7 +343,7 @@ static void parse_menu(xmlNodePtr node, gpointer data)
         goto parse_menu_fail;
 
     if (!g_hash_table_lookup(menu_hash, name)) {
-        if (!obt_xml_attr_string(node, "label", &title))
+        if (!obt_xml_attr_string_unstripped(node, "label", &title))
             goto parse_menu_fail;
 
         if ((menu = menu_new(name, title, TRUE, NULL))) {
@@ -384,6 +394,7 @@ ObMenu* menu_new(const gchar *name, const gchar *title,
     self->shortcut = parse_shortcut(title, allow_shortcut_selection,
                                     &self->title, &self->shortcut_position,
                                     &self->shortcut_always_show);
+    self->collate_key = g_utf8_collate_key(self->title, -1);
 
     g_hash_table_replace(menu_hash, self->name, self);
 
@@ -399,6 +410,7 @@ ObMenu* menu_new(const gchar *name, const gchar *title,
     self->more_menu = g_slice_new0(ObMenu);
     self->more_menu->name = _("More...");
     self->more_menu->title = _("More...");
+    self->more_menu->collate_key = "\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff";
     self->more_menu->data = data;
     self->more_menu->shortcut = g_unichar_tolower(g_utf8_get_char("M"));
 
@@ -425,6 +437,7 @@ static void menu_destroy_hash_value(ObMenu *self)
     menu_clear_entries(self);
     g_free(self->name);
     g_free(self->title);
+    g_free(self->collate_key);
     g_free(self->execute);
     g_slice_free(ObMenu, self->more_menu);
 
@@ -540,6 +553,7 @@ void menu_entry_unref(ObMenuEntry *self)
         case OB_MENU_ENTRY_TYPE_NORMAL:
             RrImageUnref(self->data.normal.icon);
             g_free(self->data.normal.label);
+            g_free(self->data.normal.collate_key);
             while (self->data.normal.actions) {
                 actions_act_unref(self->data.normal.actions->data);
                 self->data.normal.actions =
@@ -713,10 +727,13 @@ void menu_entry_set_label(ObMenuEntry *self, const gchar *label,
         break;
     case OB_MENU_ENTRY_TYPE_NORMAL:
         g_free(self->data.normal.label);
+        g_free(self->data.normal.collate_key);
         self->data.normal.shortcut =
             parse_shortcut(label, allow_shortcut, &self->data.normal.label,
                            &self->data.normal.shortcut_position,
                            &self->data.normal.shortcut_always_show);
+        self->data.normal.collate_key =
+            g_utf8_collate_key(self->data.normal.label, -1);
         break;
     default:
         g_assert_not_reached();
@@ -727,3 +744,69 @@ void menu_show_all_shortcuts(ObMenu *self, gboolean show)
 {
     self->show_all_shortcuts = show;
 }
+
+static int sort_func(const void *a, const void *b) {
+    const ObMenuEntry *e[2] = {*(ObMenuEntry**)a, *(ObMenuEntry**)b};
+    const gchar *k[2];
+    gint i;
+
+    for (i = 0; i < 2; ++i) {
+        if (e[i]->type == OB_MENU_ENTRY_TYPE_NORMAL)
+            k[i] = e[i]->data.normal.collate_key;
+        else {
+            g_assert(e[i]->type == OB_MENU_ENTRY_TYPE_SUBMENU);
+            if (e[i]->data.submenu.submenu)
+                k[i] = e[i]->data.submenu.submenu->collate_key;
+            else
+                return -1; /* arbitrary really.. the submenu doesn't exist. */
+        }
+    }
+    return strcmp(k[0], k[1]);
+}
+
+/*!
+  @param start The first entry in the range to sort.
+  @param end The last entry in the range to sort.
+*/
+static void sort_range(ObMenu *self, GList *start, GList *end, guint len)
+{
+    ObMenuEntry **ar;
+    GList *it;
+    guint i;
+    if (!len) return;
+
+    ar = g_slice_alloc(sizeof(ObMenuEntry*) * len);
+    for (i = 0, it = start; it != g_list_next(end); ++i, it = g_list_next(it))
+        ar[i] = it->data;
+    qsort(ar, len, sizeof(ObMenuEntry*), sort_func);
+    for (i = 0, it = start; it != g_list_next(end); ++i, it = g_list_next(it))
+        it->data = ar[i];
+    g_slice_free1(sizeof(ObMenuEntry*) * len, ar);
+}
+
+void menu_sort_entries(ObMenu *self)
+{
+    GList *it, *start, *end, *last;
+    guint len;
+
+    /* need the submenus to know their labels for sorting */
+    menu_find_submenus(self);
+
+    start = self->entries;
+    len = 0;
+    for (it = self->entries; it; it = g_list_next(it)) {
+        ObMenuEntry *e = it->data;
+        if (e->type == OB_MENU_ENTRY_TYPE_SEPARATOR) {
+            end = g_list_previous(it);
+            sort_range(self, start, end, len);
+
+            it = g_list_next(it); /* skip over the separator */
+            start = it;
+            len = 0;
+        }
+        else
+            len += 1;
+        last = it;
+    }
+    sort_range(self, start, last, len);
+}
