@@ -224,6 +224,7 @@ gint main(gint argc, gchar **argv)
         event_reset_time();
 
         do {
+            gchar *xml_error_string = NULL;
             ObPrompt *xmlprompt = NULL;
 
             if (reconfigure) obt_keyboard_reload();
@@ -264,6 +265,14 @@ gint main(gint argc, gchar **argv)
                 else
                     OBT_PROP_ERASE(obt_root(ob_screen), OB_CONFIG_FILE);
 
+                if (obt_xml_last_error(i)) {
+                    xml_error_string = g_strdup_printf(
+                        _("One or more XML syntax errors were found while parsing the Openbox configuration files.  See stdout for more information.  The last error seen was in file \"%s\" line %d, with message: %s"),
+                        obt_xml_last_error_file(i),
+                        obt_xml_last_error_line(i),
+                        obt_xml_last_error_message(i));
+                }
+
                 /* we're done with parsing now, kill it */
                 obt_xml_instance_unref(i);
             }
@@ -362,17 +371,12 @@ gint main(gint argc, gchar **argv)
             reconfigure = FALSE;
 
             /* look for parsing errors */
-            {
-                xmlErrorPtr e = xmlGetLastError();
-                if (e) {
-                    gchar *m;
-
-                    m = g_strdup_printf(_("One or more XML syntax errors were found while parsing the Openbox configuration files.  See stdout for more information.  The last error seen was in file \"%s\" line %d, with message: %s"), e->file, e->line, e->message);
-                    xmlprompt =
-                        prompt_show_message(m, _("Openbox Syntax Error"), _("Close"));
-                    g_free(m);
-                    xmlResetError(e);
-                }
+            if (xml_error_string) {
+                xmlprompt = prompt_show_message(xml_error_string,
+                                                _("Openbox Syntax Error"),
+                                                _("Close"));
+                g_free(xml_error_string);
+                xml_error_string = NULL;
             }
 
             g_main_loop_run(ob_main_loop);
@@ -515,9 +519,9 @@ static void print_version(void)
 {
     g_print("Openbox %s\n", PACKAGE_VERSION);
     g_print(_("Copyright (c)"));
-    g_print(" 2008        Mikael Magnusson\n");
+    g_print(" 2004   Mikael Magnusson\n");
     g_print(_("Copyright (c)"));
-    g_print(" 2003-2006   Dana Jansens\n\n");
+    g_print(" 2002   Dana Jansens\n\n");
     g_print("This program comes with ABSOLUTELY NO WARRANTY.\n");
     g_print("This is free software, and you are welcome to redistribute it\n");
     g_print("under certain conditions. See the file COPYING for details.\n\n");
@@ -576,7 +580,7 @@ static void run_startup_cmd(void)
                        G_SPAWN_SEARCH_PATH |
                        G_SPAWN_DO_NOT_REAP_CHILD,
                        NULL, NULL, NULL, &e);
-    if (!g_shell_parse_argv(startup_cmd, NULL, &argv, &e)) {
+    if (!ok) {
         g_message("Error launching startup command: %s",
                   e->message);
         g_error_free(e);
