@@ -21,22 +21,17 @@
 
 #include "misc.h"
 
-#include "render/render.h"
-#include "render/theme.h"
+#include "obrender/render.h"
+#include "obrender/theme.h"
+#include "obt/display.h"
 
 #include <glib.h>
-#include <X11/Xlib.h>
-
-struct _ObMainLoop;
 
 extern RrInstance *ob_rr_inst;
 extern RrImageCache *ob_rr_icons;
 extern RrTheme    *ob_rr_theme;
 
-extern struct _ObMainLoop *ob_main_loop;
-
-/*! The X display */
-extern Display *ob_display;
+extern GMainLoop *ob_main_loop;
 
 /*! The number of the screen on which we're running */
 extern gint     ob_screen;
@@ -50,6 +45,9 @@ extern gboolean ob_sm_restore;
 extern gboolean ob_replace_wm;
 extern gboolean ob_debug_xinerama;
 
+/*! The current locale for the LC_MESSAGES category */
+extern const gchar *ob_locale_msg;
+
 /* The state of execution of the window manager */
 ObState ob_state(void);
 void ob_set_state(ObState state);
@@ -61,10 +59,8 @@ void ob_exit_replace(void);
 
 void ob_reconfigure(void);
 
-void ob_exit_with_error(const gchar *msg);
+void ob_exit_with_error(const gchar *msg) G_GNUC_NORETURN;
 
 Cursor ob_cursor(ObCursor cursor);
 
-gboolean ob_keycode_match(KeyCode code, ObKey key);
-
 #endif
